{*******************************************************}
{                                                       }
{              Funciones Utiles                         }
{                                                       }
{   Copyright (c) 2002-2004 Pablo Alebizakis            }
{            Montevideo, Uruguay                        }
{                                                       }
{             Uselo bajo su propio riesgo               }
{             Use it under your own risk                }
{                                                       }
{*******************************************************}

unit DelphiUtiles;

interface
uses Controls, SysUtils;

function PadStr(Valor: String; Largo:Integer; Relleno:Char=' '):String;
function PadRStr(Valor: String; Largo:Integer; Relleno:Char=' '):String;
function PadCero(Valor: String; Largo:Integer):String;

function NomPropio(Texto: String):String;

function NumeroEnLetras(x:Currency):String;

Function CalcAnios(FechaIni,FechaFin:TDate):Integer;

function CICheckDigit(Cedula:LongWord):Shortint;
procedure SplitCedula(const CedulaConDigito:LongWord;var Cedula:LongWord;var Barra:ShortInt);


implementation


function Pad(Valor: String; Largo:Integer; Relleno:Char; Izda:Boolean):String;
{Rellena el resto de Valor con el caracter de Relleno hasta llegar al largo}
var
   I:Integer;
begin
  Valor:=Trim(Valor);
  I:=Largo-Length(Valor);
  If I>0 then
    If Izda then
       Result:=StringOfChar(Relleno,I)+Valor
    else
        Result:=Valor+StringOfChar(Relleno,I)
  else
    Result:=Valor;
end;

function PadStr(Valor: String; Largo:Integer; Relleno:Char=' '):String;
{Rellena el resto de Valor con el caracter de Relleno hasta llegar al largo}
begin
  Result:=Pad(Valor,Largo,Relleno,True);
end;

function PadRStr(Valor: String; Largo:Integer; Relleno:Char=' '):String;
{Rellena el resto de Valor con el caracter de Relleno hasta llegar al largo}
begin
  Result:=Pad(Valor,Largo,Relleno,False);
end;

function PadCero(Valor: String; Largo:Integer):String;
{ Devuelve el Valor con ceros a la izquierda }
begin
    Result:=PadStr(Valor,Largo,'0');
end;

function NomPropio(Texto: String):String;
{ Devuelve el texto con la primera letra en mayuscula y el resto en minusculas
 Ej. "LUIS A. DE HERRERA" = "Luis A. de Herrera"}
const
     { Letras que deben convertirse aparte.}
     exMayusc='';
     exMinusc='';
var
   I,J:Integer;
   C:String;
   Cap:Boolean;
begin
     Cap:= True;
     Result:='';
     For I:=1 To Length(Texto) do begin
         C:=Copy(Texto, I, 1);
         If Cap Then begin
            J:=Pos(C,exMinusc);
            If J>0 Then C:=exMayusc[J] else C:=UpperCase(C);
         end
         Else begin
              J:=Pos(C,exMayusc);
              If J>0 Then C:=exMinusc[J] else C:=LowerCase(C);
         end;
         Result:=Result+C;
         Cap:=Pos(Copy(Result,Length(Result),1), '''.- ')>0;
         If Cap Then Cap:=UpperCase(Copy(Texto, I + 1, 3)) <> 'DE ';
         If Cap Then Cap:=UpperCase(Copy(Texto, I + 1, 4)) <> 'DEL ';
         If Cap Then Cap:=UpperCase(Copy(Texto, I + 1, 2)) <> 'A ';
         If Cap Then Cap:=UpperCase(Copy(Texto, I + 1, 2)) <> 'Y ';
         If Cap Then Cap:=UpperCase(Copy(Texto, I + 1, 3)) <> 'UN ';
     End;
End;

function NumeroEnLetras(x:Currency):String;
{Devuelve el numero x escrito en letras}

  function Nombre(x:Currency):String;
  const
       uni :Array[0..29] of string=('','UNO','DOS','TRES','CUATRO','CINCO','SEIS','SIETE','OCHO','NUEVE','DIEZ','ONCE','DOCE','TRECE','CATORCE','QUINCE','DIECISEIS','DIECISIETE','DIECIOCHO','DIECINUEVE','VEINTE','VEINTIUNO','VEINTIDOS','VEINTITRES','VEINTICUATRO','VEINTICINCO','VEINTISEIS','VEINTISIETE','VEINTIOCHO','VEINTINUEVE');
       dec :Array[0..9] of string =('','','','TREINTA','CUARENTA','CINCUENTA','SESENTA','SETENTA','OCHENTA','NOVENTA');
       cent:Array[0..9] of string =('','CIENTO','DOSCIENTOS','TRESCIENTOS','CUATROCIENTOS','QUINIENTOS','SEISCIENTOS','SETECIENTOS','OCHOCIENTOS','NOVECIENTOS');
  var
     xcent,xdec,xuni:Integer;
  begin
     xcent:= Trunc(Int(x / 100));
     xdec:= Trunc(Int(x / 10) - 10 * xcent);
     xuni:= Trunc(x - 100 * xcent - 10 * xdec);
     If xdec > 2 Then begin
        Result:= dec[xdec];
        If xuni > 0 Then
           Result:= Result + ' Y ' + uni[xuni];
     end
     Else
         Result:= uni[xdec * 10 + xuni];
     If xcent > 0 Then Result:= cent[xcent] + ' ' + Result;
     If x = 100 Then Result:= 'CIEN';
  end;

  function EnLetras(x:Currency):String;
  var
     parte1,parte2,parte3:Currency;
     millon,millar:String;
  begin
       parte1:= Int(x / 1000000);
       parte2:= Int((x - parte1 * 1000000) / 1000);
       parte3:= x - parte1 * 1000000 - parte2 * 1000;
       If parte1 = 0 Then millon:= '';
       If parte1 = 1 Then millon:= 'UN MILLN ';
       If parte1 > 1 Then millon:= NOMBRE(parte1) + ' MILLONES ';
       If (parte1 - 100 * Int(parte1 / 100) > 11) And ((parte1 - 10 * Int(parte1 / 10)) = 1) Then
          millon:=Copy(NOMBRE(parte1), 1, Length(NOMBRE(parte1)) - 1) + ' MILLONES ';
       If parte2 = 0 Then millar:= '';
       If parte2 = 1 Then millar:= ' MIL ';
       If parte2 > 1 Then millar:= NOMBRE(parte2) + ' MIL ';
       If (parte2 - 100 * Int(parte2 / 100) > 11) And ((parte2 - 10 * Int(parte2 / 10)) = 1) Then
          millar:= Copy(NOMBRE(parte2), 1, Length(NOMBRE(parte2)) - 1) + ' MIL ';
       Result:= millon + millar + NOMBRE(parte3);
       If Int(x) = 0 Then Result:= 'CERO';
  end;

Begin
     Result:=Trim(EnLetras(Int(x)));
     If Int(x)<>x then
        Result:=Result+' CON '+ trim(EnLetras(x*100-Int(x)*100))+'/00'
     else
         Result:=Result+' CON 0/00';
end;

Function CalcAnios(FechaIni,FechaFin:TDate):Integer;
begin
  Result := Trunc((FechaFin - FechaIni) / 365.25);
end;

function CICheckDigit(Cedula:LongWord):Shortint;
{ Clculo del dgito verificador de la Cdula de Identidad Uruguaya
  Recibe la cdula sin digito verificador y devuelve el dgito.
  Si no se puede calcular el dgito devuelve -1 }
const
     Coef:Array[1..7] of Byte =(2,9,8,7,6,3,4);
var
   StrCI: String;
   I,D: Byte;
   Suma,Cociente,Resto: LongWord;
begin
     If (Cedula=0) or (Cedula > 9999999) then begin
        Result:=-1;
        exit;
     end;
     StrCI:=PadCero(IntToStr(Cedula),7);
     Suma:=0;
     For I:=1 to 7 do begin
         D:=Ord(StrCI[I])-48;
         Suma:= Suma+(D * Coef[I]);
     end;
     Cociente:=Suma div 10;
     Resto:= Suma - (Cociente * 10);
     If Resto<>0 then
        Result:=10-Resto
     else
         Result:=Resto;
end;

procedure SplitCedula(const CedulaConDigito:LongWord;var Cedula:LongWord;var Barra:ShortInt);
var
   StrCI: String;
begin
     StrCI:=IntToStr(CedulaConDigito);
     Cedula:=StrToInt(Copy(StrCI,1,Length(StrCI)-1));
     Barra:=StrToInt(StrCI[Length(StrCI)]);
end;

end.
